/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "magneto2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define MAGNETO2_DUMMY 0

#define MAGNETO2_ENABLE_FRAME_TRANSFER_START      0xAA
#define MAGNETO2_ENABLE_FRAME_TRANSFER_FOLLOWER   0xFF
#define MAGNETO2_MAX_16BIT_VALUE                  0xFFFF
#define MAGNETO2_ANGLE_COEFFICIENT                45.5

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS

static void transfer_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void magneto2_cfg_setup ( magneto2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_1;
}

MAGNETO2_RETVAL magneto2_init ( magneto2_t *ctx, magneto2_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return MAGNETO2_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, MAGNETO2_DUMMY );

    return MAGNETO2_OK;

}

uint16_t magneto2_read_data ( magneto2_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t check_value;
    uint16_t inverted_data_value;
    uint16_t data_value;

    uint8_t write_buffer_start[ 1 ];
    uint8_t write_buffer_follow[ 2 ];
    uint8_t data_value_buffer[ 2 ];
    uint8_t inverted_data_value_buffer[ 2 ];

    write_buffer_start[ 0 ] = MAGNETO2_ENABLE_FRAME_TRANSFER_START;
    write_buffer_follow[ 0 ] = MAGNETO2_ENABLE_FRAME_TRANSFER_FOLLOWER;
    write_buffer_follow[ 1 ] = MAGNETO2_ENABLE_FRAME_TRANSFER_FOLLOWER;

    block.tx_buffer = write_buffer_start;
    block.rx_buffer = 0;
    block.tx_length = 1;
    block.rx_length = 0;

    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, &block );

    block.tx_buffer = write_buffer_follow;
    block.rx_buffer = 0;
    block.tx_length = 1;
    block.rx_length = 0;
    
    spi_master_transfer( &ctx->spi, &block );

    transfer_delay();

    block.tx_buffer = write_buffer_follow;
    block.rx_buffer = data_value_buffer;
    block.tx_length = 2;
    block.rx_length = 2;

    
    spi_master_transfer( &ctx->spi, &block );

    transfer_delay();

    block.tx_buffer = write_buffer_follow;
    block.rx_buffer = inverted_data_value_buffer;
    block.tx_length = 2;
    block.rx_length = 2;

    
    spi_master_transfer( &ctx->spi, &block );
    spi_master_stop( &ctx->spi );

    data_value = data_value_buffer[ 1 ];
    data_value <<= 8;
    data_value |= data_value_buffer[ 0 ];

    check_value = data_value & 1;

    if ( check_value == 1 )
    {
        return -1;
    }
    
    inverted_data_value |= inverted_data_value_buffer[ 1 ];
    inverted_data_value <<= 8;
    inverted_data_value |= inverted_data_value_buffer[ 0 ];

    if ( data_value != inverted_data_value )
    {
        data_value >>= 2;
        return data_value;
    }
    else
    {
        return -1;
    }
}

float magneto2_read_angle ( magneto2_t* ctx )
{

    float angle;
    uint16_t data_value;

    data_value = magneto2_read_data( ctx );

    if ( data_value != MAGNETO2_MAX_16BIT_VALUE )
    {
        angle = (float)data_value;
        angle /= MAGNETO2_ANGLE_COEFFICIENT;

        return angle;
    }
    else
    {
        return -1;
    }
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void transfer_delay ( void ) 
{
    Delay_50us();
}

// ------------------------------------------------------------------------- END

